@@ -1,17 +1,16 @@
 #!/bin/false
-# Net::DHCP::Constants.pm
+# PODNAME: Net::DHCP::Constants
 # Author: Dean Hamstead, Stephan Hadinger
-
-package Net::DHCP::Constants;
-
-# standard module declaration
-use 5.8.0;
+# ABSTRACT: Constants for DHCP codes and options
 use strict;
 use warnings;
+use 5.8.0;
 
-our ( @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS, $VERSION );
+package Net::DHCP::Constants;
+$Net::DHCP::Constants::VERSION = '0.7';
+# standard module declaration
+our ( @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS );
 use Exporter;
-$VERSION = 0.695;
 @ISA     = qw(Exporter);
 
 @EXPORT = qw(MAGIC_COOKIE);
@@ -25,7 +24,8 @@ our ( %NWIP_CODES,       %REV_NWIP_CODES );
 our ( %CCC_CODES,        %REV_CCC_CODES );
 our ( %GEOCONF_CODES,    %REV_GEOCONF_CODES );
 our ( %RELAYAGENT_CODES, %REV_RELAYAGENT_CODES );
-our ( %SUBOPTION_CODES );
+our ( %RELAYAGENT_FORMATS, %REV_RELAYAGENT_FORMATS );
+our ( %SUBOPTION_CODES,  %REV_SUBOPTION_CODES );
 
 %EXPORT_TAGS = (
     dho_codes     => [ keys %DHO_CODES ],
@@ -38,10 +38,15 @@ our ( %SUBOPTION_CODES );
     ra_codes      => [ keys %RELAYAGENT_CODES ],
     dhcp_hashes   => [
         qw(
-          %DHO_CODES %REV_DHO_CODES %DHCP_MESSAGE %REV_DHCP_MESSAGE
-          %BOOTP_CODES %REV_BOOTP_CODES %NWIP_CODES %CCC_CODES
-          %HTYPE_CODES %REV_HTYPE_CODES %GEOCONF_CODES %REV_GEOCONF_CODES
-          %RELAYAGENT_CODES %REV_RELAYAGENT_CODES %SUBOPTION_CODES
+          %DHO_CODES %REV_DHO_CODES
+          %DHCP_MESSAGE %REV_DHCP_MESSAGE
+          %BOOTP_CODES %REV_BOOTP_CODES
+          %NWIP_CODES %REV_NWIP_CODES
+          %CCC_CODES %REV_CCC_CODES
+          %HTYPE_CODES %REV_HTYPE_CODES
+          %GEOCONF_CODES %REV_GEOCONF_CODES
+          %RELAYAGENT_CODES %REV_RELAYAGENT_CODES
+          %SUBOPTION_CODES %REV_SUBOPTION_CODES
           )
     ],
     dhcp_other => [
@@ -50,12 +55,18 @@ our ( %SUBOPTION_CODES );
 );
 
 @EXPORT_OK = qw(
-  %DHO_CODES %REV_DHO_CODES %DHCP_MESSAGE %REV_DHCP_MESSAGE
-  %BOOTP_CODES %REV_BOOTP_CODES %NWIP_CODES %REV_NWIP_CODES
-  %HTYPE_CODES %REV_HTYPE_CODES %CCC_CODES %REV_CCC_CODES
-  %DHO_FORMATS %GEOCONF_CODES %REV_GEOCONF_CODES
+  %DHO_CODES %REV_DHO_CODES
+  %DHCP_MESSAGE %REV_DHCP_MESSAGE
+  %BOOTP_CODES %REV_BOOTP_CODES
+  %NWIP_CODES %REV_NWIP_CODES
+  %HTYPE_CODES %REV_HTYPE_CODES
+  %CCC_CODES %REV_CCC_CODES
+  %DHO_FORMATS
+  %SUBOPTION_FORMATS
+  %GEOCONF_CODES %REV_GEOCONF_CODES
   %RELAYAGENT_CODES %REV_RELAYAGENT_CODES
-  %SUBOPTION_CODES
+  %SUBOPTION_CODES  %REV_SUBOPTION_CODES
+
 );
 Exporter::export_tags('dho_codes');
 Exporter::export_tags('dhcp_message');
@@ -65,6 +76,7 @@ Exporter::export_tags('nwip_codes');
 Exporter::export_tags('ccc_codes');
 Exporter::export_tags('geoconf_codes');
 Exporter::export_ok_tags('dhcp_other');
+Exporter::export_ok_tags('ra_codes');
 
 # MAGIC_COOKIE for DHCP (otherwise it is BOOTP)
 use constant MAGIC_COOKIE => "\x63\x82\x53\x63";
@@ -78,16 +90,17 @@ use constant DHCP_MIN_LEN           => 548;
 BEGIN {
     %BOOTP_CODES = (
         'BOOTREQUEST' => 1,
-        'BOOTREPLY'   => 2
+        'BOOTREPLY'   => 2,
     );
 
     %HTYPE_CODES = (
         'HTYPE_ETHER'   => 1,
         'HTYPE_IEEE802' => 6,
-        'HTYPE_FDDI'    => 8
+        'HTYPE_FDDI'    => 8,
+        'HTYPE_INFINIBAND' => 32,
     );
 
-    %DHO_CODES = (    # rfc 2132
+    %DHO_CODES = (    # RFC 2132
         'DHO_PAD'                                  => 0,
         'DHO_SUBNET_MASK'                          => 1,
         'DHO_TIME_OFFSET'                          => 2,
@@ -154,7 +167,7 @@ BEGIN {
         'DHO_NWIP_SUBOPTIONS'                      => 63,
         'DHO_NISV3_DOMAIN'                         => 64,
         'DHO_NISV3_SERVER'                         => 65,
-        'DHO_TFTP_SERVER'                          => 66, # actually named 'server name' by iana
+        'DHO_TFTP_SERVER'                          => 66, # actually named 'server name' by IANA
         'DHO_BOOTFILE'                             => 67,
         'DHO_MOBILE_IP_HOME_AGENT'                 => 68,
         'DHO_SMTP_SERVER'                          => 69,
@@ -244,7 +257,7 @@ BEGIN {
     # Type 53 codes...
     %DHCP_MESSAGE = (
 
-        #RFC2132
+        # RFC2132
         'DHCPDISCOVER'   => 1,
         'DHCPOFFER'      => 2,
         'DHCPREQUEST'    => 3,
@@ -303,16 +316,19 @@ BEGIN {
 
     # Type 82 sub option codes
     %RELAYAGENT_CODES = (
-        'RA_CIRCUIT_ID'         => 1,
-        'RA_REMOTE_ID'          => 2,
-        'RA_DOCSIS_CLASS'       => 4,
-        'RA_LINK_SELECTION'     => 5,
-        'RA_SUBSCRIBER_ID'      => 6,
-        'RA_RADIUS_ATTRIBUTES'  => 7,
-        'RA_AUTHENTICATION'     => 8,
-        'RA_VENDOR_INFO'        => 9,
-        'RA_FLAGS'              => 10,
-        'RA_SERVER_ID_OVERRIDE' => 11,
+        'RAI_CIRCUIT_ID'         => 1,
+        'RAI_REMOTE_ID'          => 2,
+        'RAI_AGENT_ID'           => 3,
+        'RAI_DOCSIS_CLASS'       => 4,
+        'RAI_LINK_SELECTION'     => 5,
+        'RAI_SUBSCRIBER_ID'      => 6,
+        'RAI_RADIUS_ATTRIBUTES'  => 7,
+        'RAI_AUTHENTICATION'     => 8,
+        'RAI_VENDOR_INFO'        => 9,
+        'RAI_FLAGS'              => 10,
+        'RAI_SERVER_ID_OVERRIDE' => 11,
+        'RAI_DHCPV4_VIRTUAL_SUBNET_SELECTION' => 151, # rfc 6607
+        'RAI_DHCPV4_VIRTUAL_SUBNET_SELECTION_CONTROL' => 152, # rfc 6607
     );
 }
 
@@ -340,12 +356,15 @@ use constant \%GEOCONF_CODES;
 use constant \%RELAYAGENT_CODES;
 %REV_RELAYAGENT_CODES = reverse %RELAYAGENT_CODES;
 
+use constant \%RELAYAGENT_FORMATS;
+%REV_RELAYAGENT_FORMATS = reverse %RELAYAGENT_FORMATS;
+
 #
 # Format of DHCP options : for pretty-printing
 #   void : no parameter
 #   inet : 4 bytes IP address
-#   inets : list of 4 bytess IP addresses
-#   inets2 : liste of 4 bytes IP addresses pairs (multiple of 8 bytes)
+#   inets : list of 4 bytes IP addresses
+#   inets2 : list of 4 bytes IP addresses pairs (multiple of 8 bytes)
 #   int : 4 bytes integer
 #   short : 2 bytes integer
 #   shorts : list of 2 bytes integers
@@ -421,14 +440,14 @@ our %DHO_FORMATS = (
     DHO_DHCP_REBINDING_TIME()         => 'int',
     DHO_VENDOR_CLASS_IDENTIFIER()     => 'string',
 
-    #    DHO_DHCP_CLIENT_IDENTIFIER()  => 'ids',
+    DHO_DHCP_CLIENT_IDENTIFIER()      => 'clientid',  # rfc 2132 section 9.14
     DHO_NWIP_DOMAIN_NAME()            => 'string',    # rfc 2242
 
     DHO_NWIP_SUBOPTIONS()             => 'suboptions',                    # rfc 2242
     DHO_NISV3_DOMAIN()                => 'string',
     DHO_NISV3_SERVER()                => 'string',
     DHO_TFTP_SERVER()                 => 'string',
-    DHO_BOOTFILE()                    => 'string',
+    DHO_BOOTFILE()                    => 'string', # 67
     DHO_MOBILE_IP_HOME_AGENT()        => 'inets',
     DHO_SMTP_SERVER()                 => 'inets',
     DHO_POP3_SERVER()                 => 'inets',
@@ -439,12 +458,16 @@ our %DHO_FORMATS = (
     DHO_STREETTALK_SERVER()           => 'inets',
     DHO_STDA_SERVER()                 => 'inets',
 
-    DHO_CCC                           => 'suboptions',
+    DHO_SIP_SERVERS()                 => 'sipserv',
+    DHO_CLASSLESS_STATIC_ROUTE()      => 'csr',        # rfc 3442
+    DHO_CCC                           => 'suboptions', # 122
 
-#    DHO_USER_CLASS()  => '',                        # rfc 3004
+    # While not perfect, usage is primarily as a string.  iPXE is
+    # a common use case for this option.
+    DHO_USER_CLASS()                  => 'string',     # rfc 3004
 #    DHO_FQDN()  => '',                              # draft-ietf-dhc-fqdn-option-10.txt
-#    DHO_DHCP_AGENT_OPTIONS()  => 'relays',             # rfc 3046
-    DHO_DHCP_AGENT_OPTIONS()           => 'string',    # rfc 3046
+    DHO_DHCP_AGENT_OPTIONS()           => 'suboptions', # rfc 3046
+#    DHO_DHCP_AGENT_OPTIONS()           => 'string',    # rfc 3046
     DHO_NDS_SERVERS()                  => 'inets',     # rfc 2241
     DHO_NDS_TREE_NAME()                => 'string',    # rfc 2241
     DHO_USER_AUTHENTICATION_PROTOCOL() => 'string',    # rfc 2485
@@ -452,10 +475,33 @@ our %DHO_FORMATS = (
     DHO_NAME_SERVICE_SEARCH()          => 'shorts',    # rfc 2937
     DHO_SUBNET_SELECTION()             => 'inet',      # rfc 3011
 
-    DHO_CLASSLESS_STATIC_ROUTE()      => 'csr',        # rfc 3442
     DHO_CLASSLESS_STATIC_ROUTE_MS()   => 'csr',
 );
 
+%RELAYAGENT_FORMATS = (
+        RAI_CIRCUIT_ID()       => 'hexa',
+        RAI_REMOTE_ID()        => 'hexa',
+#         RAI_AGENT_ID()           => 'hexa',
+#         RAI_DOCSIS_CLASS()       => 'hexa',
+#         RAI_LINK_SELECTION()     => 'hexa',
+#         RAI_SUBSCRIBER_ID()      => 'hexa',
+#         RAI_RADIUS_ATTRIBUTES()  => 'hexa',
+#         RAI_AUTHENTICATION()     => 'hexa',
+        RAI_VENDOR_INFO()        => 'hexa',
+#         RAI_FLAGS()              => 'hexa',
+#         RAI_SERVER_ID_OVERRIDE() => 'hexa',
+#         RAI_DHCPV4_VIRTUAL_SUBNET_SELECTION() => 'hexa',
+#         RAI_DHCPV4_VIRTUAL_SUBNET_SELECTION_CONTROL() => 'hexa',
+),
+
+our %SUBOPTION_FORMATS = (
+    $DHO_CODES{'DHO_DHCP_AGENT_OPTIONS'}    => \%RELAYAGENT_FORMATS,
+);
+
+our %REV_SUBOPTION_FORMATS = (
+    $DHO_CODES{'DHO_DHCP_AGENT_OPTIONS'}    => \%REV_RELAYAGENT_FORMATS,
+);
+
 # Links option codes with their suboption values
 %SUBOPTION_CODES = (
     $DHO_CODES{'DHO_NWIP_SUBOPTIONS'}    => \%NWIP_CODES, # option 63
@@ -464,14 +510,30 @@ our %DHO_FORMATS = (
     $DHO_CODES{'DHO_GEOCONF'}            => \%GEOCONF_CODES, # option 123
 );
 
+# Links option codes with their reverse suboption values
+%REV_SUBOPTION_CODES = (
+    $DHO_CODES{'DHO_NWIP_SUBOPTIONS'}    => \%REV_NWIP_CODES, # option 63
+    $DHO_CODES{'DHO_DHCP_AGENT_OPTIONS'} => \%REV_RELAYAGENT_CODES, # option 82
+    $DHO_CODES{'DHO_CCC'}                => \%REV_CCC_CODES,  # option 122
+    $DHO_CODES{'DHO_GEOCONF'}            => \%REV_GEOCONF_CODES, # option 123
+);
+
 1;
 
+__END__
+
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Net::DHCP::Constants - Constants for DHCP codes and options
 
+=head1 VERSION
+
+version 0.7
+
 =head1 SYNOPSIS
 
   use Net::DHCP::Constants;
@@ -531,7 +593,7 @@ Import all DHCP Message codes.
 
 Nb. Previously Cisco used 13 for DHCPLEASEQUERY. If you need to decode
 or encode packets to communicate with such a system, you might simply
-us the integer rather than the constant - or use the updated constant
+use the integer rather than the constant - or use the updated constant
 and comment in your code appropriately.
 
 =item * dho_codes
@@ -694,6 +756,49 @@ Import all CableLabs Client Configuration
   (009) CCC_SECURITY_TICKET_CONTROL
   (010) CCC_KDC_SERVER
 
+=item * ra_codes
+
+Import all DHCP Agenet (aka Relay Agent) Codes
+
+  (001) RAI_CIRCUIT_ID
+  (002) RAI_REMOTE_ID
+  (004) RAI_DOCSIS_CLASS
+  (005) RAI_LINK_SELECTION
+  (006) RAI_SUBSCRIBER_ID
+  (007) RAI_RADIUS_ATTRIBUTES
+  (008) RAI_AUTHENTICATION
+  (009) RAI_VENDOR_INFO
+  (010) RAI_FLAGS
+  (011) RAI_SERVER_ID_OVERRIDE
+  (151) RAI_DHCPV4_VIRTUAL_SUBNET_SELECTION
+  (152) RAI_DHCPV4_VIRTUAL_SUBNET_SELECTION_CONTROL
+
+=item * CABLELABS_OPTION43
+
+ (002) DEVICE_TYPE
+3 ECM
+4 SERIAL_NUMBER
+5 HW_VERSION
+6 SW_VERSION
+7 BOOT_ROM_VERSION
+8 OUI
+9 MODEL_NUMBER
+10 VENDOR_NAME
+11 PS_WAN_MAN_DATA
+12 CM_PS_SYSTEM_DESCRIPTION
+13 CM_PS_FIRMWARE_REV
+14 FIREWALL_POLICY_FILE_VERSION
+15 ESAFES
+16 DEVICE_TPID
+17 DEVICE SMID
+18 VIDEOSECURITYELEMENT
+31 MTA_MAC_ADDRESS
+32 MTA_CORRELATION_ID
+51 CARD_VENDOR_NAME
+52 CARD_CAPABILITY
+53 CARD_VENDOR_DEVICE_ID
+54 CARD_ID
+
 =back
 
 =head1 TO DO, LIMITATIONS
@@ -714,17 +819,14 @@ L<Net::DHCP::Packet>, L<Net::DHCP::Options>
 
 =head1 AUTHOR
 
-Dean Hamstead E<lt>djzort@cpan.orgE<gt> and past Authors (see README)
+Dean Hamstead <dean@fragfest.com.au>
 
-=head1 GOT PATCHES?
+=head1 COPYRIGHT AND LICENSE
 
-Many young people like to use Github, so by all means send me pull requests at
+This software is Copyright (c) 2022 by Dean Hamstead.
 
-https://github.com/djzort/Net-DHCP
+This is free software, licensed under:
 
-=head1 COPYRIGHT
-
-This program is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself.
+  The MIT (X11) License
 
 =cut
